/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* utility JavaScript for the demo pages */

define(["dojo/_base/declare",
		"dojo/_base/lang",
		"dojo/_base/xhr",
		"dojo/dom-construct",
		"dijit/layout/ContentPane",
		"dijit/form/TextBox"],
		function(declare, lang, xhr, domconstruct, ContentPane, TextBox){
				 	
	function sizeIframeToContent(ifnode, fullwidth){
		// NB this will not work cross-domain
		// include 10px fudge factor in case of stray borders/frames
		// NB scrollWidth comes out as "1" on FF, so use parentNode.scrollWidth which, curiously, is set OK
		ifnode.style.width = fullwidth ? "100%" : ((ifnode.contentDocument || ifnode.contentWindow.document).body.parentNode.scrollWidth) + "px";
		ifnode.style.height = ((ifnode.contentDocument || ifnode.contentWindow.document).body.parentNode.scrollHeight + 10) + "px";
	}

 	return declare("idx.demos._PageProxyContentPane", [ContentPane], {
		// pageProxyContentPane is a ContentPane with an extra property, iframesrc,
		// which is the URL which will be loaded into an IFRAME and used as the
		// pane content. Loading is lazy, begun on first show. The IFRAME is sized
		// to the content size (NB this will not work cross-domain).
	 	_onShow: function(){
			this.inherited(arguments);
			
			if(this.iframesrc || this.mobilesrc){
				var basesrc = this.iframesrc || this.mobilesrc,
					src = basesrc,
					mob = !this.iframesrc;
				delete this.iframesrc;
				delete this.mobilesrc;
				
				this._setContent(this.onDownloadStart(), true);
				
				var ifrmholder = this.containerNode;
				
				if(mob){
					ifrmholder = domconstruct.create("div", { style: "display: none;" }, this.containerNode);
					
					if(!this.width){
						this.width = "720";
					}
					if(!this.height){
						this.height = "540";
					}
					if(!this.theme){
						this.theme = "Android";
					}
					
					src = basesrc + "?width=" + this.width + "&height=" + this.height + "&theme=" + this.theme; 
				}
				
				var ifrm = domconstruct.create("iframe", {
					frameBorder: "0",
					scrolling: "no",
					src: src,
					title: this.title,
					style: "width: 1px; height: 1px; display: none;" + (mob ? "border: 12px solid #131313; border-radius: 6px;" : "")
				}, ifrmholder);
				
				var onLoadHandler = dojo.hitch(this, function(event){
					// when loading is complete, remove all but the last
					// child node (which will be our IFRAME) and then
					// resize the iframe to the content
					while (this.containerNode.firstChild.nextSibling) {
						this.containerNode.removeChild(this.containerNode.firstChild);
					}
					sizeIframeToContent(event.target || event.srcElement, !mob);
					ifrmholder.style.display = "block";
					ifrm.style.display = "block";
				});
				if(ifrm.addEventListener){
					ifrm.addEventListener("load", onLoadHandler, false);
				}else{
					ifrm.attachEvent("onload", onLoadHandler);
				}
				
				if(mob && (this.showControls !== false)){
					var panel = domconstruct.create("div", {}, ifrmholder, "first");
					
						/*,
							subpanel1 = domconstruct.create("div", { style: "display: inline-block;" }, panel),
							android
						<input type="radio" id="pp_theme_android" data-dojo-type="dijit/form/RadioButton" checked value="Android"/> <label for="pp_theme_android">Android theme</label> <br />
			    		<input type="radio" id="pp_theme_ios" data-dojo-type="dijit/form/RadioButton" value="iPhone"/> <label for="pp_theme_ios">iOS (iPhone/iPad) theme</label>		
					
					</div>
	
					<div style="display: inline-block; width: 200px; position: relative; min-height: 40px; padding: 0 1em;">
	
						<div style="position: absolute; left: 79px; top: 10px; height: 30px; width: 3px; background-color: darkGray;"></div>		
						<span style="float: left; font-size: 9px; padding-bottom: 3px;">smaller screen</span>
						<span style="float: right; font-size: 9px; padding-bottom: 3px;">larger screen</span>
						<div data-dojo-type="dijit/form/HorizontalRule" data-dojo-props="container: 'topDecoration', count: 5" style="clear: both; height: 1px; margin: 0 11px;"></div>
						
						<input id="hslider" type="range" value="3" data-dojo-type="dijit/form/HorizontalSlider" data-dojo-props="minimum: 0, maximum: 4, showButtons: false, discreteValues: 5" />
						
						<div data-dojo-type="dijit/form/HorizontalRule" data-dojo-props="container: 'bottomDecoration', count: 5" style="height: 3px; margin: 0 11px;"></div>
						<span style="position: absolute; right: 126px; text-align: right; font-size: 10px; color: darkGray;"><span style="font-size: 14px;">&larr;</span> &ldquo;phone&rdquo;</span>
						<span style="position: absolute; left: 86px; font-size: 10px; color: darkGray;">&ldquo;tablet&rdquo; <span style="font-size: 14px;">&rarr;</span></span>
						
					</div>
	
					<div style="display: inline-block;">
					
						<input type="radio" id="pp_orientation_portrait" data-dojo-type="dijit/form/RadioButton" checked value="portrait"/> <label for="pp_orientation_portrait">portrait orientation</label> <br />
			    		<input type="radio" id="pp_orientation_landscape" data-dojo-type="dijit/form/RadioButton" value="landscape"/> <label for="pp_orientation_portrait">landscape orientation</label>		
					
					</div>*/
					
					var largest = Math.max(this.width, this.height),
						sz = "Small phone";
					if(largest >= 900){
						sz = "Large tablet";
					}else if (largest >= 700){
						sz = "Medium tablet";
					}else if (largest >= 500){
						sz = "Small tablet";
					}else if (largest >= 400){
						sz = "Large phone";
					}	
					
					panel.innerHTML = "Theme variant: <b>" + this.theme + "</b>" +
									  ", screen size: <b>" + sz + "</b>" +
									  ", orientation: <b>" + ((this.width > this.height) ? "landscape" : "portrait") + "</b>" +
									  " (<a href='" + ifrm.src + "' target='_new'>view this configuration in a new tab</a>)" +
									  "<p class='demoSampleNotice'><b>Note:</b> this sample gives users of desktop browsers " +
									  "an impression of how IDX One UI mobile would look on a mobile device. However, factors " +
									  "such as fonts, pixel ratios, browser features and CSS rendering mean that the effect " +
									  "will not be exactly the same as on the appropriate device. In particular, there will be " +
									  "extentsive differences when this is viewed using a non-WebKit browser, such as Firefox " +
									  "or Microsoft Internet Explorer, and so use of a WebKit browser, such as Google Chrome or " +
									  "Apple Safari, is recommended. If you are viewing this on a mobile device (phone or tablet), " +
									  "you may wish to <a href='" + basesrc + "' target='_new'>view this sample directly</a>.</p>"
				}
			}
										
			if(this.textsrc){
				var src = this.textsrc;
				delete this.textsrc;
				
				this._setContent(this.onDownloadStart(), true);

				var dfd = xhr.get({
					url: src,
					handleAs: "text"
				});
				
				dfd.addCallback(dojo.hitch(this, function(data){
					this.destroyDescendants();
					
					var pre = document.createElement("PRE");
					pre.className = "codePage";
					pre.appendChild(document.createTextNode(data));
					this.containerNode.appendChild(pre);
				}));
			}							
		}
	});
			
});