/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/_base/sniff",
	"dojo/_base/lang",
	"dojo/_base/event",
	"dojo/dom-style",
	"dijit/form/Textarea",
	"../util",
	"./_CssStateMixin",
	"./_ValidationMixin",
	"./_CompositeMixin",
	"dojo/text!./templates/Textarea.html"
], function(declare, lang, array, has, lang, event, domStyle, Textarea, iUtil, _CssStateMixin, _ValidationMixin, _CompositeMixin, template){
	var iForm = lang.getObject("idx.oneui.form", true); // for backward compatibility with IDX 1.2
	
    /**
	 * @name idx.form.Textarea
	 * @class idx.form.Textarea is a composite widget which enhanced dijit.form.Textarea with following features:
	 * <ul>
	 * <li>Built-in label</li>
	 * <li>Built-in label positioning</li>
	 * <li>Built-in hint</li>
	 * <li>Built-in hint positioning</li>
	 * <li>Built-in required attribute</li>
	 * <li>Built-in validation support</li>
	 * <li>One UI theme support</li>
	 * </ul>
	 * @augments dijit.form.Textarea
	 * @augments idx.form._CssStateMixin
	 * @augments idx.form._CompositeMixin
	 * @augments idx.form._ValidationMixin
	 */
	return iForm.Textarea = declare("idx.form.Textarea", [Textarea, _CompositeMixin, _ValidationMixin, _CssStateMixin], 
	/**@lends idx.form.Textarea.prototype*/
	{
		instantValidate: false,
		templateString: template,
		baseClass: "idxTextareaWrap",
		oneuiBaseClass: "dijitTextBox dijitTextArea dijitExpandingTextArea",
		
		postCreate: function(){
			this.extension = {
				"input" : "_onInput",
				"blur" 	: "_onBlur",
				"focus" : "_onFocus"
			}
			this.inherited(arguments);
			array.forEach(array.filter(this._connects, function(conn){ 
				return conn && conn[0] && conn[0][1] == "onfocus"; 
			}), this.disconnect, this);
		},
		_isEmpty: function(){
			return (this.trim ? /^\s*$/ : /^$/).test(this.get("value")); // Boolean
		},
		_setFieldWidthAttr: function(){
			this.inherited(arguments);
			var width = parseInt(this.oneuiBaseNode.style.width);
			var widthInPx = iUtil.normalizedLength(width);
			if(has("ie") <= 7){
				domStyle.set(this.oneuiBaseNode, "width", widthInPx - 3 + "px");
			}else{
				domStyle.set(this.oneuiBaseNode, "width", widthInPx + 2 + "px");
			}
			this.resize();
		},
		_isValidFocusNode: function(mousedownNode){
			return (this.hintPosition == "inside" && mousedownNode == this._phspan || 
				mousedownNode == this.oneuiBaseNode.parentNode) || this.inherited(arguments);
		}
	});
});
